declare module BA.Ui.Controls {
    /**
     * Events that get triggered in context with the list import
     **/
    class ListImportEvents {
        /** constant value: prefix "FieldMapping_" **/
        static readonly FieldMappingPrefix: string;
        /** constant value: suffix "_SubTableField" **/
        static readonly SubTableFieldSuffix: string;
        /** constant value: suffix "_ParentField" **/
        static readonly ParentFieldSuffix: string;
        /**
        * Checks whether the dropped objects actually may be dropped on a address list import ribbon action.
        *
        * Two situations can be handled here:
        *   1. There are no files (i.e. when a div or some image from another part of the page is dragged and dropped here)
        *   2. There are files whose names do not end in '.csv'
        *
        * If dropping is allowed, the function will return 'null', in all other cases it will return an error message to
        * be displayed to the user.
        *
        * Hint: due to security reasons this can actually only be called in onDrop, so the framework will call this
        *   function first, when the user has already dropped data on the ribbon item, and depending on the outcom
        *   call OnDrop() afterwards.
        *
        * @param e The original drag-event object.
        * @returns Returns an error message (or null if drop is allowed).
        **/
        static MayDrop(e: DragEvent): string;
        /**
         * This function is invoked when (anything) is dropped on a address list import ribbon action.
         * It collects the dropped files and sends it to the server (via ajax call) and if that completed successfully,
         * it will call the list import dialog.
         *
         * Hint: Sadly, due to security reasons, we cannot access any information about the objects being dragged in a
         * dragOver event... in that case we could simply forbid to drop anything other than csv files (with such a
         * nice forbidden sign at the mouse cursor), but this is not possible with files here (no access to file name
         * or other content-information in dragOver/enter/whatever). But as it is, the framework will fist call
         * the mayDrop-function and depending on its results this function here.
         **/
        static OnDrop(e: DragEvent, customData: CustomData): void;
        /**
         * Presets values of the controls on the field mapping dialog (also called on refresh).
         **/
        static InitFieldMappingDialog(sender: BAClientPopupControl): void;
        /**
         * This is called when a combobox content is changed by the user. It re-serializes the current dialog
         * settings / contents and under certain circumstances refreshes the dialog so an additional control can be shown.
         *
         * @param sender The control that has been changed.
         * @param evt The change event.
         **/
        static FieldMappingTargetFieldOnChange(sender: BAClientComboBox, evt: ASPxClientProcessingModeEventArgs): void;
        /**
         * This function is called when the list import dialog is submitted using its 'Ok' button.
         * It is called multiple times recursively.
         * At first, the field mapping dialog is about to be called called because the value of the hidden
         * mappingControl input field is empty or null. In this branch, it first checks whether a converter is selected
         * and if not, it emits an error message and returns control to the user. In this case, the mapping dialog is
         * NOT invoked.
         * If everything is all right, it constructs the dialog parameters for the field mapping dialog and calls it.
         * Cancelling the field mapping dialog will cancel the whole process, submitting it will set the returned
         * result into the previously empty mappingControl input field and call this function again.
         * Now, with mappingControl not being empty, it is checked whether the selected converter needs further
         * information and if applicable the converter dialog is called. The converter dialog is NOT provided any
         * data from the past user interaction. Any information that is returned by the converter dialog is
         * serialized and put into the AdditionalConverterParametersJson input control (and the function is called once
         * again). Here, too, all processing is cancelled, if the converter dialog does not return a result with the
         * okButton as status.
         * At the end, if all needed information is collected and all control fields are set to their values, handling
         * is passed to the server side handleAction method of the list import dialog.
         **/
        static ListImportDialogOK(sender: Dialog.BADialogButton, evt: ASPxClientButtonClickEventArgs): void;
        /**
         * Called when the field mapping dialog is closed using its ok button.
         * It modifies the mapping information slightly in order to fit the serverside's needs and returns the mappings
         * as dialog result data, then closes the dialog.
         **/
        static ListImportFieldMappingDialogOK(sender: ASPxClientButton, evt: any): void;
        /**
         * Called if processing should be canceled.
         * Closes the list import dialog with cancel button result.
         **/
        private static CancelImportDialog;
        /**
         * Returns the current settings as object.
         **/
        private static GetCurrentSettings;
        /**
         * Returns the input element with the given name (without DLG prefix) from the current dialog.
         **/
        private static GetControlByName;
    }
}
