declare module BA.Ui.Event {
    /**
     *  Function-interface for the internal event handler (not exposed).
     *
     *  @template T Type of the data parameter of the handler function (defaults to CustomData)
     **/
    interface BAInternalEventHandlerFunction<T = CustomData> {
        (event: JQueryEventObject, data: T): boolean;
    }
    /**
     * Function-interface for general event handler functions.
     *
     * Example:
     * let handler: BAEventHandlerFunction<CustomData> = (data: CustomData) => { return true; };
     *
     * @template T Type of the data parameter of the handler function (defaults to CustomData)
     **/
    export interface BAEventHandlerFunction<T = CustomData> {
        (data: T): boolean;
    }
    /** Constants for standard event types */
    export class StandardEventTypes {
        /** Type of event triggered when a field is changed */
        static readonly FieldUpdate: string;
        /** Type of event triggered when a new tab is opening (aborts if handler returns false) */
        static readonly TabOpening: string;
        /** Type of event triggered when a new tab was opened */
        static readonly TabOpened: string;
        /** Type of event triggered when an existing tab was activated */
        static readonly TabActivated: string;
        /** Type of event triggered when an existing tab should be closed (aborts if handler returns false) */
        static readonly TabClosing: string;
        /** Type of event triggered when an existing tab has been closed */
        static readonly TabClosed: string;
        /** Type of event triggered when a new dialog has been opened */
        static readonly DialogOpened: string;
        /** Type of event triggered when an existing dialog has been closed */
        static readonly DialogClosed: string;
        /** Type of event triggered when some record is saved in the ui (aborts if handler returns false) */
        static readonly RecordSaving: string;
        /** Type of event triggered when a record has been saved sucessfully in the ui */
        static readonly RecordSaved: string;
        /** Type of event triggered when the 'refresh grids' ribbon bar action is clicked */
        static readonly RefreshGrids: string;
        /** Type of event triggered when attachments refresh */
        static readonly AttachmentsRefresh: string;
        /** Type of event triggered when widgets change their provided parameter */
        static readonly WidgetParameterUpdated: string;
        /** Type of event triggered when there is a global authentication error in google maps */
        static readonly GlobalGMapsError: string;
    }
    /**
     * Manages the ui events used in the application. Provides different levels of API and is instantiated in 'top.eventManager'.
     * The API consists of a low-level set of functions for adding, removing and triggering events of any type, a public set of static functions
     * for the same purpose but easier to use and a public set of static funtions for handling field updates in particular.
     **/
    export class EventManager {
        private _eventTarget;
        private _handlerMap;
        /** Cleans up the map of stored handlers-by-id and removes handlers that were created in a context that is no longer valid. */
        private CleanUpHandlerMap;
        /** Creates a new instance of the event manager and initializes the internal JQuery event target. */
        constructor();
        /**
         * This function is internal and should only used for low-level event handling or if the task cannot be done using the public API.
         * It adds an event listener for the given type. The listener/handler may contain a scope which will be taken into account here.
         * The handler object is expected to have an id. It is checked whether this id is already registered. You cannot register more than
         * one handler with the same id, independant of type and scope. The id must be unique over all handlers.
         * Adding a handler with an already registered id will lead to that corresponding handler being deregistered first.
         *
         * @param handler Handler object that must contain a unique id and a context window and may contain a scope for this event.
         */
        addEventHandler<T = CustomData>(handler: InternalEventHandler<T>): void;
        /**
         * This function is internal and should only used for low-level event handling or if the task cannot be done using the public API.
         * It removes a previously registered handler by using its id (thus the handler does not need to be exactly the same object like
         * in most other event dispatching systems).
         *
         * @param handler Handler object that must contain a unique id and the original scope for the event(s) to remove.
         * @returns (boolean) true, if the handler could be removed successfully
         */
        removeEventHandler<T = CustomData>(id: string): boolean;
        /**
         * This function is internal and should only used for low-level event handling or if the task cannot be done using the public API.
         * It triggers all event handlers that have been registered for the given type and optionally scope.
         *
         * HINT: If no scope is given, all events of that type containing a scope will be triggered nevertheless!
         *
         * @param type Type of the event that should be triggered
         * @param scope Scope of the event that should be triggered (optional)
         * @param data Data to be passed to the handler function
         * @returns (boolean) True, if the handler succeeded in doing its job, false if not.
         *                  False usually means, that the handler asks the caller to stop whatever it is going to do.
         */
        triggerEvent(type: string, scope: string, data: Object): boolean;
        /**
         * Adds a new event handler for the given type using the given id. The id must be globally unique and not be null or empty!
         * You cannot add more than one event handler with the same id without removing it first!
         * The event manager supports event scopes, which is similar to some sort of event category. The scope may be omitted.
         *
         * Example:
         * let handler: BAEventHandlerFunction = (data: CustomData) => { handleSomething(); return true; };
         * EventManager.AddEventHandler('myevent', 'my_unique_id', handler, activeTab.Id.toString());
         *
         * Hint: The public API only supports CustomData as data for handlers. If you need anything else, you will have to stick to the internal API (not recommended).
         *
         * @param type The name of the event to add. This can either be one of the built-in events (like 'click') or simply anything for a custom event.
         * @param id The id the event is registered under. This must be unique (ui-wide!).
         * @param handler Function that takes a CustomData object as parameter.
         * @param scope The scope of the event. This could be i.e. the current tab id or anything else. (optional)
         */
        static AddEventHandler(type: string, id: string, handler: BAEventHandlerFunction<CustomData>, scope?: string): void;
        /**
         * Removes a previously registered event from the event handling system.
         * You cannot remove an event that has not been registered before!
         *
         * @param id The id the event was registered under.
         * @returns (boolean) true, if the handler was removed successfully
         */
        static RemoveEventHandler(id: string): boolean;
        /**
         * Executes all event handlers of the given type and scope. Scope may always be omitted here in order to call ALL handlers of the
         * given type, regardless of their scope.
         * This function will always return true until an executed handler function returns false, also, if no handlers where executed at all.
         *
         * @param type The name of the event to trigger. This can either be one of the built-in events (like 'click') or simply anything for a custom event.
         * @param scope The scope of the event. This could be i.e. the current tab id or anything else. (optional)
         * @param data CustomData object that will be passed to the handler function for further information.
         * @returns (boolean) True, if the handler succeeded in doing its job (or if it does not exist), false if not.
         *                  False usually means, that the handler asks the caller to stop whatever it is going to do.
         */
        static TriggerEvent(type: string, scope?: string, data?: CustomData): boolean;
        /**
         * Adds a special handler for field updates to the event manager that will be triggered by any field (most fields) that is changed in the ui.
         * A scope is required. The handler will be passed an object of tyhe UpdateFieldEventData on execution.
         * You can register multiple handlers for field changes using this function.
         *
         * Example:
         * let handler: BAEventHandlerFunction<UpdateFieldEventData> = (data: UpdateFieldEventData) => { console.log(data.fieldName + ' changed!'); return true;};
         * EventManager.AddFieldUpdateHandler('Subject', handler, activeTabId);
         *
         * Hint: Most fields in the ui will fire this event when they are changed and lose focus.
         *
         * @param fieldName Name of the field (OrmFieldName) or the guid of the relation definition (for relation fields)
         * @param handler Handler function that is called when the event is triggered
         * @param scope The scope of the event. This could be i.e. the current tab id or anything else. (required here)
         * @returns (string) Returns the generated id for this event registration. Keep this if you might need to remove the event again!
         */
        static AddFieldUpdateHandler(fieldName: string, handler: BAEventHandlerFunction<UpdateFieldEventData>, scope: string): string;
        /**
         * Removes field change event handlers. You will need at least the field name and the scope if you want to remove any event handler.
         * If you need to remove a concrete event handler, you will need its id (as returned by AddFieldUpdateHandler). Omitting the id will
         * remove all field change events for the given field and scope.
         *
         * @param fieldName Name of the field (OrmFieldName) or the guid of the relation definition (for relation fields)
         * @param scope The scope of the event. This could be i.e. the current tab id or anything else. (required here)
         * @param id The id of the concrete event to remove or null for removing all events of this field and scope
         * @returns (boolean) true, if the handler(s) could be removed successfully
         */
        static RemoveFieldUpdateHandler(fieldName: string, scope: string, id?: string): boolean;
        /**
         * Triggers the event handlers for the given field and scope.
         *
         * @param fieldName Name of the field (OrmFieldName) or the guid of the relation definition (for relation fields)
         * @param scope The scope of the event. This could be i.e. the current tab id or anything else. (required here)
         * @param fullFieldName Used for fields located on sub records; while in this case the fieldName property will only contain
         *                  i. e. 'Address.City', the fullFieldName will include the index of the sub record (i. e. Addresses[1].City).
         * @param additionalData additional data that will be added to the event data
         * @returns (boolean) True, if the handler succeeded in doing its job (or if it does not exist), false if not.
         *                  False usually means, that the handler asks the caller to stop whatever it is going to do.
         */
        static TriggerFieldUpdated(fieldName: string, scope: string, fullFieldName?: string, additionalData?: CustomData): boolean;
        /**
         * Clears all events from a given scope.
         *
         * @param scope the scope to clear the events from
         */
        static ClearAllEvents(scope: string): void;
    }
    /**
     * Simple container class that contains the following information:
     * - Type : the type of the fired event
     * - FieldName : name of the field that triggered the event (possibly simplyfied on sub records like 'Addresses.City')
     * - Context : the window object where the event took place and the parent object of the handler function
     * - Scope : the event scope (usually the tab guid)
     * - FullFieldName : only used for sub records (contains the index like 'Addresses[1].City'), null else
     * - AdditionalData : a CustomData object with additional data to be delivered to the event handler
     */
    export class UpdateFieldEventData {
        Type: string;
        FieldName: string;
        Context: Window;
        Scope: string;
        FullFieldName: string;
        AdditionalData: CustomData;
        constructor(Type: string, FieldName: string, Context: Window, Scope: string, FullFieldName: string, AdditionalData: CustomData);
    }
    /**
     * Simple container class (base class) that contains the following information (not exposed):
     * - Id : the event's id
     * - Scope : the event's scope (optional)
     * - Options : (reserved)
     * - Context : the window object where the event took place and the parent object of the handler function
     **/
    class InternalEventHandlerBase {
        Type: string;
        Id: string;
        Scope: string;
        Options: CustomData;
        Context: Window;
        constructor(Type: string, Id: string, Scope: string, Options?: CustomData, Context?: Window);
    }
    /**
     * Simple container class (extends InternalEventHandlerBase) that contains the following ADDITIONAL information:
     * - Handler : handler function to call: (data: T) => {...}
     *
     * @template T type of the handler function data parameter (defaults to CustomData)
     **/
    export class InternalEventHandler<T = CustomData> extends InternalEventHandlerBase {
        Handler: BAEventHandlerFunction<T>;
        HandlerFunctionInternal: BAInternalEventHandlerFunction<T>;
        constructor(Type: string, Id: string, Scope: string, Handler?: BAEventHandlerFunction<T>, Options?: CustomData, Context?: Window);
    }
    export {};
}
